@@ -1,5 +1,5 @@
 ;; mc-pgp.el, PGP support for Mailcrypt
-;; Copyright (C) 1995  Jin Choi <jsc@mit.edu>
+;; Copyright (C) 1995  Jin Choi <jin@atype.com>
 ;;                     Patrick LoPresti <patl@lcs.mit.edu>
 
 ;;{{{ Licensing
@@ -51,7 +51,7 @@ use its default.")
   "Regular expression matching an error from PGP")
 (defconst mc-pgp-sigok-re "^.*Good signature.*"
   "Regular expression matching a PGP signature validation message")
-(defconst mc-pgp-newkey-re "^[ \t]*\\(No\\|[0-9]+\\) +new.*"
+(defconst mc-pgp-newkey-re "^[ \t]*\\(No\\|[0-9]+\\) +new [ku].*"
   "Regular expression matching a PGP key snarf message")
 (defconst mc-pgp-nokey-re
   "Cannot find the public key matching userid '\\(.+\\)'$"
@@ -68,14 +68,15 @@ use its default.")
 	    (obuf (current-buffer)))
 	(unwind-protect
 	    (progn
-	      (call-process mc-pgp-path nil buffer nil
-			    "+verbose=1" "-kv" "XXXXXXXXXX")
+	      (call-process mc-pgp-path nil buffer nil "+verbose=1"
+			    "+language=en" "-kv" "XXXXXXXXXX")
 	      (set-buffer buffer)
 	      (goto-char (point-min))
 	      (re-search-forward "^Key ring:\\s *'\\(.*\\)'")
 	      (setq mc-pgp-keydir
 		    (file-name-directory
-		     (buffer-substring (match-beginning 1) (match-end 1)))))
+		     (buffer-substring-no-properties
+		      (match-beginning 1) (match-end 1)))))
 	  (set-buffer obuf)
 	  (kill-buffer buffer))))
   mc-pgp-keydir)
@@ -100,17 +101,18 @@ PGP ID.")
 	  (unwind-protect
 	      (progn
 		(setq buffer (generate-new-buffer " *mailcrypt temp"))
-		(call-process mc-pgp-path nil buffer nil "-kv" str keyring)
+		(call-process mc-pgp-path nil buffer nil
+			      "+language=en" "-kv" str keyring)
 		(set-buffer buffer)
 		(goto-char (point-min))
 		(if (re-search-forward key-regexp nil t)
 		    (progn
 		      (setq result
-			    (cons (buffer-substring
+			    (cons (buffer-substring-no-properties
 				   (match-beginning 3) (match-end 3))
 				  (concat
 				   "0x"
-				   (buffer-substring
+				   (buffer-substring-no-properties
 				    (match-beginning 2) (match-end 2)))))
 		      (setq mc-pgp-key-cache (cons (cons str result)
 						   mc-pgp-key-cache)))))
@@ -149,10 +151,8 @@ PGP ID.")
 	(mc-pgp-always-sign mc-pgp-always-sign)
 	(obuf (current-buffer))
 	args key passwd result pgp-id)
-    (setq key (mc-pgp-lookup-key (or id mc-pgp-user-id)))
-    (if mc-encrypt-for-me
-	(setq recipients (cons (cdr key) recipients)))
-    (setq args (list "+encrypttoself=off +verbose=1" "+batchmode" "-feat"))
+    (setq args (list "+encrypttoself=off +verbose=1" "+batchmode"
+		     "+language=en" "-feat"))
     (if mc-pgp-comment
 	(setq args (cons (format "+comment=%s" mc-pgp-comment) args)))
     (if mc-pgp-alternate-keyring
@@ -162,6 +162,7 @@ PGP ID.")
 	     (or mc-pgp-always-sign sign (y-or-n-p "Sign the message? ")))
 	(progn
 	  (setq mc-pgp-always-sign t)
+	  (setq key (mc-pgp-lookup-key (or id mc-pgp-user-id)))
 	  (setq passwd
 		(mc-activate-passwd
 		 (cdr key)
@@ -172,6 +173,12 @@ PGP ID.")
 	  (setq msg (format "Encrypting+signing as %s ..." (car key))))
       (setq mc-pgp-always-sign 'never))
 
+    (or key
+	(setq key (mc-pgp-lookup-key mc-pgp-user-id)))
+
+    (if mc-encrypt-for-me
+	(setq recipients (cons (cdr key) recipients)))
+
     (setq args (append args recipients))
     
     (message "%s" msg)
@@ -183,7 +190,8 @@ PGP ID.")
       (if (re-search-forward mc-pgp-nokey-re nil t)
 	  (progn
 	    (if result (error "This should never happen."))
-	    (setq pgp-id (buffer-substring (match-beginning 1) (match-end 1)))
+	    (setq pgp-id (buffer-substring-no-properties
+			  (match-beginning 1) (match-end 1)))
 	    (and
 	     (y-or-n-p
 	      (format "Key for '%s' not found; try to fetch? " pgp-id))
@@ -196,26 +204,25 @@ PGP ID.")
 	  t)))))
 
 (defun mc-pgp-decrypt-parser (result)
-  (let (start end)
-    (goto-char (point-min))
-    (cond ((eq result 0)
-	   ;; Valid signature
-	   (re-search-forward "^Signature made.*\n")
-	   (if (looking-at
-		"\a\nWARNING:  Because this public key.*\n.*\n.*\n")
-	       (goto-char (match-end 0)))
-	   (cons (point) (point-max)))
-	  ((eq result 1)
-	   (re-search-forward
-	    "\\(File is conventionally encrypted\\. *\\)?Just a moment\\.+")
-	   (if (eq (match-beginning 1) (match-end 1))
-	       (if (looking-at
-		    "\nFile has signature.*\\(\n\a.*\n\\)*\nWARNING:.*\n")
-		   (goto-char (match-end 0)))
-	     (if (looking-at "Pass phrase appears good\\. \\.")
-		 (goto-char (match-end 0))))
-	   (cons (point) (point-max)))
-	  (t nil))))
+  (goto-char (point-min))
+  (cond ((eq result 0)
+	 ;; Valid signature
+	 (re-search-forward "^Signature made.*\n")
+	 (if (looking-at
+	      "\a\nWARNING:  Because this public key.*\n.*\n.*\n")
+	     (goto-char (match-end 0)))
+	 (cons (point) (point-max)))
+	((eq result 1)
+	 (re-search-forward
+	  "\\(File is conventionally encrypted\\. *\\)?Just a moment\\.+")
+	 (if (eq (match-beginning 1) (match-end 1))
+	     (if (looking-at
+		  "\nFile has signature.*\\(\n\a.*\n\\)*\nWARNING:.*\n")
+		 (goto-char (match-end 0)))
+	   (if (looking-at "Pass phrase appears good\\. \\.")
+	       (goto-char (match-end 0))))
+	 (cons (point) (point-max)))
+	(t nil)))
 
 (defun mc-pgp-decrypt-region (start end &optional id)
   ;; returns a pair (SUCCEEDED . VERIFIED) where SUCCEEDED is t if
@@ -224,6 +231,7 @@ PGP ID.")
 	(buffer (get-buffer-create mc-buffer-name))
 	(obuf (current-buffer))
 	args key new-key passwd result pgp-id)
+    (undo-boundary)
     (setq key (mc-pgp-lookup-key (or id mc-pgp-user-id)))
     (setq
      passwd
@@ -235,7 +243,7 @@ PGP ID.")
        (mc-activate-passwd id "PGP passphrase for conventional decryption: ")))
     (if passwd
 	(setenv "PGPPASSFD" "0"))
-    (setq args '("+verbose=1" "+batchmode" "-f"))
+    (setq args '("+verbose=1" "+batchmode" "+language=en" "-f"))
     (if mc-pgp-alternate-keyring
 	(setq args (append args (list (format "+pubring=%s"
 					      mc-pgp-alternate-keyring)))))
@@ -250,20 +258,20 @@ PGP ID.")
       (save-excursion
 	(set-buffer buffer)
 	(goto-char (point-min))
-	(if (and
-	     (re-search-forward mc-pgp-key-expected-re nil t)
-	     (setq pgp-id (concat "0x" (buffer-substring (match-beginning 1)
-							 (match-end 1))))
-	     (y-or-n-p
-	      (format "Key %s not found; attempt to fetch? " pgp-id))
-	     (mc-pgp-fetch-key (cons nil pgp-id)))
-	    (progn
-	      (set-buffer obuf)
-	      (undo-start)
-	      (undo-more 1)
-	      (mc-pgp-decrypt-region start end id))
-	  (mc-message mc-pgp-key-expected-re buffer)
-	  (cons t (eq result 0)))))
+	(if (re-search-forward mc-pgp-key-expected-re nil t)
+	    (setq pgp-id (concat "0x" (buffer-substring-no-properties
+				       (match-beginning 1)
+				       (match-end 1))))))
+      (if (and pgp-id
+	       (y-or-n-p
+		(format "Key %s not found; attempt to fetch? " pgp-id))
+	       (mc-pgp-fetch-key (cons nil pgp-id)))
+	  (progn
+	    (undo-start)
+	    (undo-more 1)
+	    (mc-pgp-decrypt-region start end id))
+	(mc-message mc-pgp-key-expected-re buffer)
+	(cons t (eq result 0))))
      ;; Decryption failed; maybe we need to use a different user-id
      ((and
        (set-buffer buffer)
@@ -272,8 +280,8 @@ PGP ID.")
 	"^Key for user ID:.*\n.*Key ID \\([0-9A-F]+\\)" nil t)
        (setq new-key
 	     (mc-pgp-lookup-key
-	      (concat "0x" (buffer-substring (match-beginning 1)
-					     (match-end 1)))))
+	      (concat "0x" (buffer-substring-no-properties (match-beginning 1)
+							   (match-end 1)))))
        (not (and id (equal key new-key))))
       (set-buffer obuf)
       (mc-pgp-decrypt-region start end (cdr new-key)))
@@ -304,7 +312,7 @@ PGP ID.")
     (setenv "PGPPASSFD" "0")
     (setq args
 	  (list
-	   "-fast" "+verbose=1"
+	   "-fast" "+verbose=1" "+language=en"
 	    (format "+clearsig=%s" (if unclear "off" "on"))
 	    "+batchmode" "-u" (cdr key)))
     (if mc-pgp-comment
@@ -333,7 +341,7 @@ PGP ID.")
   (let ((buffer (get-buffer-create mc-buffer-name))
 	(obuf (current-buffer))
 	args pgp-id)
-    (setq args '("+verbose=1" "+batchmode" "-f"))
+    (setq args '("+verbose=1" "+batchmode" "+language=en" "-f"))
     (if mc-pgp-alternate-keyring
 	(setq args (append args (list (format "+pubring=%s"
 					      mc-pgp-alternate-keyring)))))
@@ -347,21 +355,23 @@ PGP ID.")
 	(if (and
 	     (not no-fetch)
 	     (re-search-forward mc-pgp-key-expected-re nil t)
-	     (setq pgp-id (concat "0x" (buffer-substring (match-beginning 1)
-							 (match-end 1))))
+	     (setq pgp-id
+		   (concat "0x" (buffer-substring-no-properties
+				 (match-beginning 1)
+				 (match-end 1))))
 	     (y-or-n-p
 	      (format "Key %s not found; attempt to fetch? " pgp-id))
 	     (mc-pgp-fetch-key (cons nil pgp-id))
 	     (set-buffer obuf))
 	    (mc-pgp-verify-region start end t)
-	  (mc-message mc-pgp-error-re buffer))))))
-	    
+	  (mc-message mc-pgp-error-re buffer)
+	  nil)))))
 
 (defun mc-pgp-insert-public-key (&optional id)
   (let ((buffer (get-buffer-create mc-buffer-name))
 	args)
     (setq id (or id mc-pgp-user-id))
-    (setq args (list "+verbose=1" "+batchmode" "-kxaf" id))
+    (setq args (list "+verbose=1" "+batchmode" "+language=en" "-kxaf" id))
     (if mc-pgp-comment
 	(setq args (cons (format "+comment=%s" mc-pgp-comment) args)))
     (if mc-pgp-alternate-keyring
@@ -380,7 +390,7 @@ PGP ID.")
 (defun mc-pgp-snarf-keys (start end)
   ;; Returns number of keys found.
   (let ((buffer (get-buffer-create mc-buffer-name)) tmpstr args)
-    (setq args '("+verbose=1" "+batchmode" "-kaf"))
+    (setq args '("+verbose=1" "+batchmode" "+language=en" "-kaf"))
     (if mc-pgp-alternate-keyring
 	(setq args (append args (list (format "+pubring=%s"
 					      mc-pgp-alternate-keyring)))))
@@ -393,8 +403,9 @@ PGP ID.")
 	  (if (re-search-forward mc-pgp-newkey-re nil t)
 	      (progn
 		(if mc-pgp-display-snarf-output (mc-display-buffer buffer))
-		(setq tmpstr (buffer-substring (match-beginning 1) 
-					       (match-end 1)))
+		(setq tmpstr (buffer-substring-no-properties
+			      (match-beginning 1) 
+			      (match-end 1)))
 		(if (equal tmpstr "No")
 		    0
 		  (car (read-from-string tmpstr))))))
@@ -424,7 +435,7 @@ PGP ID.")
   "/htbin/pks-extract-key.pl?op=get&search=%s"
   "The URL to pass to the keyserver")
 
-(defvar mc-pgp-keyserver-address "www-swiss.ai.mit.edu"
+(defvar mc-pgp-keyserver-address "pgp.ai.mit.edu"
   "Host name of keyserver")
 
 (defvar mc-pgp-keyserver-port 80
@@ -446,7 +457,7 @@ when fetching keys.")
       (and (re-search-forward mc-pgp-key-begin-line nil t)
 	   (setq start (match-beginning 0))
 	   (re-search-forward mc-pgp-key-end-line nil t)
-	   (buffer-substring start (match-end 0))))))
+	   (buffer-substring-no-properties start (match-end 0))))))
 
 (defun mc-pgp-fetch-from-keyrings (id)
   (let ((keyring-list mc-pgp-fetch-keyring-list)
@@ -540,7 +551,7 @@ is not necessarily an error, since we may have merely fired off an Email
 request for the key."
   (interactive)
   (let ((methods mc-pgp-fetch-methods)
-	(process-connection-type nil) key proc buf win args)
+	(process-connection-type nil) key proc buf args)
     (if (null id)
 	(setq id (cons (read-string "Fetch key for: ") nil)))
     (while (and (not key) methods)
@@ -554,7 +565,7 @@ request for the key."
       (unwind-protect
 	  (save-window-excursion
 	    (setq buf (generate-new-buffer " *PGP Key Info*"))
-	    (setq win (pop-to-buffer buf))
+	    (pop-to-buffer buf)
 	    (if (< (window-height) (/ (frame-height) 2))
 		(enlarge-window (- (/ (frame-height) 2)
 				   (window-height))))
